Documentation
¶
Index ¶
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
Types ¶
type Config ¶
type Config struct {
// Expected network delay. Used as a unit for ticks.
NetworkDelay StringDuration
// Timeout after value was proposed, replica will wait for Timeout
// before executing fallback with classic paxos
Timeout int
// Each observer for the same subject must reinforce other observer vote after reinforce timeout.
ReinforceTimeout int
// Connectivity is a K paramter, used for monitoring topology construction.
// Each node will have K observers and K subjects.
Connectivity int
LowWatermark int
HighWatermark int
// Network
BroadcastFanout int
RetryPeriod StringDuration
DialTimeout, SendTimeout StringDuration
// Bootstrap
JoinTries int
JoinTimeout StringDuration
Seed *types.Node
IP string
Port uint64
}
type Rapid ¶
type Rapid struct {
// contains filtered or unexported fields
}
func (Rapid) Configuration ¶
func (r Rapid) Configuration() (*types.Configuration, <-chan struct{})
type StringDuration ¶
func (StringDuration) MarshalJSON ¶
func (d StringDuration) MarshalJSON() ([]byte, error)
func (*StringDuration) UnmarshalJSON ¶
func (d *StringDuration) UnmarshalJSON(b []byte) error
Click to show internal directories.
Click to hide internal directories.